declare namespace Base {
	type True<Value = any> = { 1: Value }
	type False<Value = any> = { 0: Value }
}

export namespace Logic {
	type True = keyof Base.True
	type False = keyof Base.False
	type Statements<TrueStatement, FalseStatement> = Base.True<TrueStatement> & Base.False<FalseStatement>
	type Contradiction = True & False | undefined
	type Boolean = True | False
	type Not<Condition extends Boolean> = Statements<False, True>[Condition]
	type Some<Conditions extends Boolean, Value extends Boolean = True> = Has<Conditions, Value>
	type Every<Conditions extends Boolean, Value extends Boolean = True> = Not<Some<Conditions, Not<Value>>>
	type And<ConditionA extends Boolean, ConditionB extends Boolean> = Every<ConditionA | ConditionB>
	type Or<ConditionA extends Boolean, ConditionB extends Boolean> = Some<ConditionA | ConditionB>
	type If<Condition extends Boolean, Then, Else> = Statements<Then, Else>[Condition]
}

type Has<Union extends string, Key extends string> = ({
	[Key in Union]: Logic.True
} & {
	[Key: string]: Logic.False
})[Key]
